/*
 * mdBlinky.cpp
 * 
 * Version 0.1, 2018-05-24.
 * 
 * by Michel Deslierres 
 *
 * See the accompanying file license.txt for the
 * conditions of use.
 * 
 */

#include "mdBlinky.h"

mdBlinky::mdBlinky(uint8_t pin, ledActive_t ledActive) //constructor
{
  pin_ = pin;
  if (ledActive) {
    led_on_ = HIGH;
  } else {
    led_on_ = LOW;   
  }
  led_off_ = 1 - led_on_;
  
  pinMode(pin_, OUTPUT);
  digitalWrite(pin_, led_off_);
  blinkState_ = LED_OFF;
  blinkMode_ = OFF;
}

//Set attributes

void mdBlinky::setMode(blinkMode_t value) {
  blinkMode_ = value;
  if (blinkMode_ == OFF) {
    blinkState_ = LED_OFF;
    digitalWrite(pin_, led_off_);
  } else if (blinkMode_ == ON) {
    blinkState_ = LED_ON;
    digitalWrite(pin_, led_on_);
  } else if (blinks_ == 0) {
      if (offTime_ == 0) {
        blinkState_ = BLINK_OFF;
        digitalWrite(pin_, led_off_);
      } else {
        blinkState_ = BLINK_ON;
        digitalWrite(pin_, led_on_);
      }
  } else {  
    digitalWrite(pin_, led_on_);
    blinkState_ = BLINK_ON;
    blinkCounter_ = blinks_;
    cycleCounter_ = cycles_;
    eventTime_ = millis();
  }
}

void mdBlinky::setBlinks(uint16_t value) { blinks_ = value;};
void mdBlinky::setBlinkOffTime(uint16_t value) { blinkOffTime_ = value;};
void mdBlinky::setBlinkOnTime(uint16_t value) { blinkOnTime_ = value;};
void mdBlinky::setOffTime(uint16_t value) { offTime_ = value;};
void mdBlinky::setCycles(uint16_t value) { cycles_ = value;};

void mdBlinky::setParams(uint16_t blinks, uint16_t blinkOffTime, uint16_t blinkOnTime, uint16_t offTime, uint16_t cycles) {
  blinks_ = blinks;
  blinkOffTime_ = blinkOffTime;
  blinkOnTime_ = blinkOnTime;
  offTime_ = offTime;
  cycles_ = cycles;    
  setMode(BLINK);
};

void mdBlinky::setParams(uint16_t params_array[], uint8_t index, uint16_t cycles) {
  int n = index*4;
  blinks_ = params_array[n];
  blinkOffTime_ = params_array[n+1];
  blinkOnTime_ = params_array[n+2];
  offTime_ = params_array[n+3];
  cycles_ = cycles;    
  setMode(BLINK);
  /*
  Serial.print("blinks: "); Serial.println(blinks_);
  Serial.print("blinkOffTime: "); Serial.println(blinkOffTime_);
  Serial.print("blinkOnTime: "); Serial.println(blinkOnTime_);
  Serial.print("OffTime: "); Serial.println(offTime_);
  */
}




blinkMode_t mdBlinky::getMode(void) { return blinkMode_; };
    
boolean mdBlinky::blink() {
  if (blinkState_ == LED_OFF) {
    return false; 
    
  } else if (blinkState_ == LED_ON) {
    return true;
    
  } else if (blinkState_ == BLINK_ON) {
    if (blinks_ == 0) {
      return true;
    }
    if ( (millis() - eventTime_) > blinkOnTime_ ) {
      blinkCounter_ -= 1;
       digitalWrite(pin_, led_off_);
       if (blinkCounter_ < 1) {
        blinkState_ = OFF_TIME;
      } else {
        blinkState_ = BLINK_OFF;
      }
      eventTime_ = millis();
      return false;
    }
    return true;
    
  } else if (blinkState_ == BLINK_OFF) {
    if (blinks_ == 0) {
      return false;
    }
    if ( (millis() - eventTime_) > blinkOffTime_ ) {
      digitalWrite(pin_, led_on_);
      blinkState_ = BLINK_ON;
      eventTime_ = millis();
      return true;
    }
    return true;

  } else if (blinkState_ == OFF_TIME) {
    if ( (millis() - eventTime_) > (unsigned long)(offTime_) ) {
      if (cycleCounter_ == 1) {
        blinkState_ = LED_OFF;
        blinkMode_ = OFF;
        return false;
      } else if (cycleCounter_ > 1) {
        cycleCounter_ -= 1;
      }     
      digitalWrite(pin_, led_on_);
      blinkState_ = BLINK_ON;
      blinkCounter_ = blinks_;
      eventTime_ = millis();
      return true;
    }
    return false;  
  }
}

